<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class ExportacaoBalancaToledo {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function find($CDFILIAL, $CDTABEPREC, $LISTA, $TODOS, $PRODINI, $PRODFIN, $IDPESAPROD, $IDTABEPRECGER = null) {
        $enableGeneralPriceTable = false;
        
        if ($IDTABEPRECGER == "%S%" || $IDTABEPRECGER == "S") {
            $enableGeneralPriceTable = true;
        }
        
        $params = array(
         	":CDFILIAL" => $CDFILIAL,
         	":CDTABEPREC" => $CDTABEPREC,
         	":LISTA" => $LISTA,
         	":TODOS" => $TODOS,
         	":PRODINI" => $PRODINI,
         	":PRODFIN" => $PRODFIN,
         	":IDPESAPROD" => $IDPESAPROD
 		); 
        
        if ($enableGeneralPriceTable) {
             return $this->entityManager->getConnection()->fetchAll(self::GET_TXT_BALANCA_GERAL, $params);
         } 
         
        return $this->entityManager->getConnection()->fetchAll(self::GET_TXT_BALANCA, $params);
    }
    
    public function findTabelaPrecoGeral($conditions) {
        $params = array(
            ":CDFILIAL" => $conditions['CDFILIAL'],
            ":NRORG"    => $conditions['NRORG']
        );
        
        return $this->entityManager->getConnection()->fetchAll(self::GET_TAB_PRECO_GERAL, $params);
    }
    
    const GET_TAB_PRECO_GERAL = "
        SELECT T.CDTABEPREC, T.NMTABEPREC, T.NRORG
            FROM FILITABGER F
                JOIN TABEPRECGER T
                    ON T.CDTABEPREC = F.CDTABEPREC
                        AND T.NRORG = F.NRORG
            WHERE F.CDFILIAL = :CDFILIAL
                AND T.NRORG = :NRORG
        ORDER BY T.CDTABEPREC
    ";
    
    public function saveInfoExtra ($rows){
        $params = array(
            ':CDINFOEXTRA'         => $rows[0]['CDINFOEXTRA'],
            ':DSINFOEXTRA'         => $rows[0]['DSINFOEXTRA'],
            ':DSOBSINFOEXTRA'      => $rows[0]['DSOBSINFOEXTRA'],
            ':NRORG'               => $rows[0]['NRORG']
 		);
 		$this->entityManager->getConnection()->executeQuery(self::INSERT_INFOEXTRA, $params);
    }
    
    const INSERT_INFOEXTRA = "
	    INSERT INTO INFOEXTRA
		  (CDINFOEXTRA, DSINFOEXTRA, DSOBSINFOEXTRA, NRORG)
		  VALUES (:CDINFOEXTRA, :DSINFOEXTRA, :DSOBSINFOEXTRA, :NRORG)
    ";
    
    public function updateInfoExtra ($rows){
        $params = array(
            ':CDINFOEXTRA'         => $rows[0]['CDINFOEXTRA'],
            ':DSINFOEXTRA'         => $rows[0]['DSINFOEXTRA'],
            ':DSOBSINFOEXTRA'      => $rows[0]['DSOBSINFOEXTRA'],
            ':NRORG'               => $rows[0]['NRORG']
 		);
 		
 		$this->entityManager->getConnection()->executeQuery(self::UPDATE_INFOEXTRA, $params);
    }
    
    const UPDATE_INFOEXTRA = "
	     UPDATE INFOEXTRA
    	  SET
    	    DSINFOEXTRA 	= :DSINFOEXTRA,
    	    DSOBSINFOEXTRA 	= :DSOBSINFOEXTRA
    	  WHERE CDINFOEXTRA = :CDINFOEXTRA
    	   AND NRORG			= :NRORG
    ";
    
    public function deleteInfoExtra($rows) {
        $params = array(
            ':CDINFOEXTRA'          => $rows[0]['CDINFOEXTRA'],
            ':DSINFOEXTRA'          => $rows[0]['DSINFOEXTRA'],
            ':DSOBSINFOEXTRA'       => $rows[0]['DSOBSINFOEXTRA'],
            ':NRORG'                => $rows[0]['NRORG']
        );
 		return $this->entityManager->getConnection()->executeQuery(self::DELETE_INFOEXTRA, $params);
    }
    
   const DELETE_INFOEXTRA = "
        DELETE FROM INFOEXTRA 
        WHERE NRORG = :NRORG 
        AND CDINFOEXTRA = :CDINFOEXTRA
    ";
    
    public function exportTXTINFO() {
		return $this->entityManager->getConnection()->fetchAll(self::GET_TXTINFO);
	}

	/* Salva Cadastramento Automatico
     * @param $row
     * @throws \Exception
     */
    public function salvaCadastramentoAutomatico($row){
        try {

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $params = array(
             	":P_CDFILIAL"    => $row['CDFILIAL'],
             	":P_CDINFOEXTRA" => $row['CDINFOEXTRA'],
             	":P_TODOS"       => ($row['INTERVALISTA'] != "T") ? null : $row['INTERVALISTA'],
             	":P_LISTA"       => !empty($row['LISTA']) ? implode('_', $row['LISTA']) : null,
             	":P_PRODINI"     => $row['CDARVPRODI'],
             	":P_PRODFIN"     => $row['CDARVPRODF'],
             	":P_CDTABEPREC"  => $row['CDTABEPREC']
     		);
     		
     		if($row['IDTABEPRECGER'] == "%S%" || $row['IDTABEPRECGER'] == "S") {
     		    $this->entityManager->getConnection()->executeQuery(self::UPDATE_CADAUTOMATICO_PRODUTO_GERAL, $params);
     		} else {
     		    $this->entityManager->getConnection()->executeQuery(self::UPDATE_CADAUTOMATICO_PRODUTO, $params);
     		}
     		
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    const GET_TXT_BALANCA = "
        SELECT '01'||';'|| DECODE(IDPESAPROD,'N','1','0') ||';'|| CASE WHEN LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) > 6 THEN
        		SUBSTR(NVL(CDATAPRODITM,CDBARPRODUTO), LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) - 5, LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)))
        		WHEN LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) < 6 THEN LPAD(NVL(NVL(CDATAPRODITM,CDBARPRODUTO),0),6,0)
        		ELSE NVL(CDATAPRODITM,CDBARPRODUTO)
        		END ||';'|| TRIM(REPLACE(TO_CHAR(NVL(A.VRPRECITEM,0),'0000.00'),'.','')) ||';'|| LPAD(NVL(QTDIAVALPRODVND,0),3,0)||';'|| RPAD(NVL(NMPRODUTO,'*'),50,'*') ||';'||
        		RPAD(NVL(CDINFOEXTRA,0),6,0) ||';'|| '0000' ||';'|| '000000' ||';'|| '1' ||';'|| DECODE(IDIMPDTEMBALAGEM, 'N', '0', '1') ||';'|| '0000' ||';'|| '000000000000'||';'|| '00000000000'||';'|| '1' ||';'||
        		'0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '000000000000' ||';'|| '000000' ||';'|| '|01|' ||';'||
        		RPAD('*',35,'*') ||';'|| RPAD('*',35,'*') ||';'|| '000000' ||';'|| '000000' ||';'|| '000000' ||';'||  '000000' ||';'|| '0' ||';'|| '||' ||';'|| '0' ||';'|| '||'
        		||';' AS TEXTO,
        		IDPESAPROD,		CDATAPRODITM,		CDBARPRODUTO,		A.VRPRECITEM,		QTDIAVALPRODVND,		NMPRODUTO,		CDINFOEXTRA,
        		:CDFILIAL AS CDFILIAL, :CDTABEPREC AS CDTABEPREC, :LISTA AS LISTA, :TODOS AS TODOS, :PRODINI AS PRODINI, :PRODFIN AS PRODFIN

        FROM PRODUTO P
        LEFT JOIN (
        			SELECT DISTINCT I.CDIDENTBUTON, I.CDATAPRODITM
        			FROM ITMENUCONFTE I, CONFTELA C
        			WHERE I.IDTPBUTTON IN ('1','9','A')
        			AND I.CDFILIAL   = :CDFILIAL
        			AND I.CDFILIAL   = C.CDFILIAL
        			AND I.NRCONFTELA = C.NRCONFTELA
        			AND I.CDATAPRODITM IS NOT NULL) I
        	   ON P.CDPRODUTO = I.CDIDENTBUTON,
        	   (SELECT I.CDPRODUTO, I.VRPRECITEM
        	   	  FROM ITEMPRECO I, TABEVEND T
        	   	 WHERE :CDFILIAL = T.CDFILIAL
        	   	 AND T.CDTABEPREC  = :CDTABEPREC
        	   	 AND (SYSDATE BETWEEN T.DTINIVGPREC AND T.DTFINVGPREC)
        	   	 AND T.CDFILIAL  = I.CDFILIAL
        	   	 AND T.CDTABEPREC  = I.CDTABEPREC
        	   	 AND T.DTINIVGPREC = I.DTINIVGPREC)A
        WHERE P.CDPRODUTO = A.CDPRODUTO
          AND ((P.IDPESAPROD = :IDPESAPROD) OR (:IDPESAPROD = 'T'))
          AND (P.CDBARPRODUTO IS NOT NULL OR I.CDATAPRODITM IS NOT NULL)
          AND ((INSTR(:LISTA, CDARVPROD)>0) OR (:TODOS = 'T') OR (CDARVPROD BETWEEN :PRODINI AND :PRODFIN))
    ";
    
    const GET_TXT_BALANCA_GERAL = "
        SELECT '01'||';'|| DECODE(IDPESAPROD,'N','1','0') ||';'|| CASE WHEN LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) > 6 THEN
        		SUBSTR(NVL(CDATAPRODITM,CDBARPRODUTO), LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) - 5, LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)))
        		WHEN LENGTH(NVL(CDATAPRODITM,CDBARPRODUTO)) < 6 THEN LPAD(NVL(NVL(CDATAPRODITM,CDBARPRODUTO),0),6,0)
        		ELSE NVL(CDATAPRODITM,CDBARPRODUTO)
        		END ||';'|| TRIM(REPLACE(TO_CHAR(NVL(B.VRPRECITEM,0),'00.00'),'.','')) ||';'|| LPAD(NVL(QTDIAVALPRODVND,0),3,0)||';'|| RPAD(NVL(NMPRODUTO,'*'),50,'*') ||';'||
        		RPAD(NVL(CDINFOEXTRA,0),6,0) ||';'|| '0000' ||';'|| '000000' ||';'|| '1' ||';'|| DECODE(IDIMPDTEMBALAGEM, 'N', '0', '1') ||';'|| '0000' ||';'|| '000000000000'||';'|| '00000000000'||';'|| '1' ||';'||
        		'0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '0000' ||';'|| '000000000000' ||';'|| '000000' ||';'|| '|01|' ||';'||
        		RPAD('*',35,'*') ||';'|| RPAD('*',35,'*') ||';'|| '000000' ||';'|| '000000' ||';'|| '000000' ||';'||  '000000' ||';'|| '0' ||';'|| '||' ||';'|| '0' ||';'|| '||'
        		||';' AS TEXTO,
        		IDPESAPROD,		CDATAPRODITM,		CDBARPRODUTO,		 B.VRPRECITEM,		QTDIAVALPRODVND,    NMPRODUTO,		CDINFOEXTRA,
        		:CDFILIAL AS CDFILIAL, :CDTABEPREC AS CDTABEPREC, :LISTA AS LISTA, :TODOS AS TODOS, :PRODINI AS PRODINI, :PRODFIN AS PRODFIN

        FROM PRODUTO P
        LEFT JOIN (
            SELECT DISTINCT I.CDIDENTBUTON, I.CDATAPRODITM
            FROM ITMENUCONFTE I, CONFTELA C
            WHERE I.IDTPBUTTON IN ('1','9','A')
              AND I.CDFILIAL = :CDFILIAL
              AND I.CDFILIAL = C.CDFILIAL
              AND I.NRCONFTELA = C.NRCONFTELA
              AND I.CDATAPRODITM IS NOT NULL
        ) I ON P.CDPRODUTO = I.CDIDENTBUTON,
        
        (
            SELECT IM.CDPRODUTO, IM.VRPRECITEM
            FROM ITEMPRECOGER IM, TABEVENDGER T
            WHERE 
              T.CDTABEPREC IN (
                SELECT CDTABEPREC FROM FILITABGER WHERE CDFILIAL = :CDFILIAL
              )
              AND (SYSDATE BETWEEN T.DTINIVGPREC AND T.DTFINVGPREC)
              AND T.CDTABEPREC = IM.CDTABEPREC
              AND T.DTINIVGPREC = IM.DTINIVGPREC
        ) B
        
        WHERE P.CDPRODUTO = B.CDPRODUTO
          AND ((P.IDPESAPROD = :IDPESAPROD) OR (:IDPESAPROD = 'T'))
          AND (P.CDBARPRODUTO IS NOT NULL OR I.CDATAPRODITM IS NOT NULL)
          AND ((INSTR(:LISTA, CDARVPROD) > 0) OR (:TODOS = 'T') OR (CDARVPROD BETWEEN :PRODINI AND :PRODFIN))
    ";

    const GET_TXTINFO = "
        SELECT CDINFOEXTRA||';'|| RPAD(NVL(DSOBSINFOEXTRA,'*'),100,'*') ||';'|| RPAD(NVL(DSINFOEXTRA,'*'),1120,'*') ||';' AS TEXTO  FROM INFOEXTRA
    ";

    const UPDATE_CADAUTOMATICO_PRODUTO = "
        UPDATE PRODUTO
            SET CDINFOEXTRA = :P_CDINFOEXTRA
            WHERE ((INSTR(:P_LISTA, CDARVPROD) > 0) OR (:P_TODOS = 'T') OR (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN))
            AND CDPRODUTO IN (
                SELECT CDPRODUTO
                    FROM ITEMPRECO I
                        JOIN TABEVEND T
                            ON T.CDFILIAL = I.CDFILIAL
                                AND T.CDTABEPREC = I.CDTABEPREC
                                AND T.DTINIVGPREC = I.DTINIVGPREC
                    WHERE T.CDFILIAL = :P_CDFILIAL
                        AND T.CDTABEPREC = :P_CDTABEPREC
                        AND (SYSDATE BETWEEN T.DTINIVGPREC AND T.DTFINVGPREC)
            )
    ";
    
    const UPDATE_CADAUTOMATICO_PRODUTO_GERAL = "
        UPDATE PRODUTO
            SET CDINFOEXTRA = :P_CDINFOEXTRA
            WHERE ((INSTR(:P_LISTA, CDARVPROD) > 0) OR (:P_TODOS = 'T') OR (CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIN))
            AND CDPRODUTO IN (
                SELECT CDPRODUTO
                    FROM ITEMPRECOGER I
                        JOIN TABEVENDGER T
                            ON T.CDTABEPREC = I.CDTABEPREC
                                AND T.DTINIVGPREC = I.DTINIVGPREC
                    WHERE T.CDTABEPREC = :P_CDTABEPREC
                        AND (SYSDATE BETWEEN T.DTINIVGPREC AND T.DTFINVGPREC)
            )
    ";
}

